unit WinMonImuUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons,
  GL, GLU, UrickGL, U3Dpolys;

type
  TWinMonImu = class(TForm)
    Panel1: TPanel;
    Label2: TLabel;
    FileName: TEdit;
    SpeedButton1: TSpeedButton;
    Label3: TLabel;
    CheckBox1: TCheckBox;
    SaveDialog1: TSaveDialog;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    Timer1: TTimer;
    Panel2: TPanel;
    Monitor: TMemo;
    Panel3: TPanel;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure CheckBox5Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
  private
    function normal(const x1, y1, z1, x2, y2, z2, x3, y3, z3: real): tpoint;
    { Private declarations }
  public
    { Public declarations }
    F: TFileStream;
    Revroll: integer;
    RevPitch: integer;
    RevYaw: integer;
    useEKF: boolean;
    YawOn: boolean;

    YawRefPos: single;
    splitList: TStringList;
    Scene:TSceneGL;
    cube: Tentity;
    light: Tlight;
    rx,ry,rz:single;

  end;

var
  WinMonImu: TWinMonImu;

implementation

uses MAIN;

{$R *.dfm}

function TWinMonImu.normal(const x1,y1,z1,x2,y2,z2,x3,y3,z3:real):tpoint;
var
  dx1,dy1,dz1,dx2,dy2,dz2:real;
  m:real;
  nx,ny,nz:real;
begin
  dx1:=x1-x2;
  dy1:=y1-y2;
  dz1:=z1-z2;
  dx2:=x2-x3;
  dy2:=y2-y3;
  dz2:=z2-z3;
  nx:=dy1*dz2-dz1*dy2;
  ny:=dz1*dx2-dx1*dz2;
  nz:=dx1*dy2-dy1*dx2;
  m:=sqrt(sqr(nx)+sqr(ny)+sqr(nz));
  if m<>0.0 then
    begin
      nx:=nx/m;
      ny:=ny/m;
      nz:=nz/m;
    end;
  result.x:=-nx;
  result.y:=-ny;
  result.z:=-nz;
end;


procedure TWinMonImu.FormCreate(Sender: TObject);
var
  Face:TFace;
  n: TPoint;
  x,y,z: real;
begin
  Revroll:=1;
  RevPitch:=1;
  RevYaw:=1;

  YawRefPos:=0;
  splitList:= TStringList.Create;
  Scene:=TSceneGL.create; //   
  Cube:=TEntity.create; //    TEntity
  Cube.SetColor(255,200,230); //   R,G,B
  Face:=cube.addFace;

  //  1- face      6  (   )

    x:=3;
    y:=0.2;
    z:=1;


    Face.AddVertex(x, y, z    ,0.0, 0.0, 1.0);  //  1-  vertex
    Face.AddVertex(-x, y, z   ,0.0, 0.0, 1.0); //  2-  vertex
    Face.AddVertex(-x, -y, z  ,0.0, 0.0, 1.0);//  3-  vertex
    Face.AddVertex(x, -y, z   ,0.0, 0.0, 1.0);//  4-  vertex


  Face:=cube.addFace;
    Face.AddVertex(x, y, -z,0.0, 0.0, -1.0);
    Face.AddVertex(x, -y, -z,0.0, 0.0, -1.0);
    Face.AddVertex(-x, -y, -z,0.0, 0.0, -1.0);
    Face.AddVertex(-x, y, -z,0.0, 0.0, -1.0);

  Face:=cube.addFace;
    Face.AddVertex(-x, y, z,-1.0, 0.0, 0.0);
    Face.AddVertex(-x, y, -z,-1.0, 0.0, 0.0);
    Face.AddVertex(-x, -y, -z,-1.0, 0.0, 0.0);
    Face.AddVertex(-x, -y, z,-1.0, 0.0, 0.0);

  Face:=cube.addFace;
    Face.AddVertex(x, y, z,1.0, 0.0, 0.0);
    Face.AddVertex(x, -y, z,1.0, 0.0, 0.0);
    Face.AddVertex(x, -y, -z,1.0, 0.0, 0.0);
    Face.AddVertex(x, y, -z,1.0, 0.0, 0.0);

  Face:=cube.addFace;
    Face.SetColor(200,10,10);
    Face.AddVertex(-x, y, -z,0.0, 1.0, 0.0);
    Face.AddVertex(-x, y, z,0.0, 1.0, 0.0);
    Face.AddVertex(x, y, z,0.0, 1.0, 0.0);
    Face.AddVertex(x, y, -z,0.0, 1.0, 0.0);

  Face:=cube.addFace;
    Face.SetColor(10,200,10);

    Face.AddVertex(-x, -y, -z,0.0, -1.0, 0.0);
    Face.AddVertex(x, -y, -z,0.0, -1.0, 0.0);
    Face.AddVertex(x, -y, z,0.0, -1.0, 0.0);
    Face.AddVertex(-x, -y, z,0.0, -1.0, 0.0);

  with cube do
  begin
    move(0,0,-15); //     x, y, z
    Rotate(-30,-30,-30); //    
  end;

  Scene.Entities.add(cube);//    

  light:=Tlight.create(2); //    
  Scene.lights.add(light); //    

  Scene.InitRC(panel3.handle); //  Handle Panel1  ,
                               //     
  Scene.UpdateArea(panel3.width,panel3.height);


end;

procedure TWinMonImu.FormDestroy(Sender: TObject);
begin
  if Timer1.Enabled then begin
    MainForm.AddToOutput('mon off ekf'+#13#10);
    Timer1.Enabled:=false;
    if f<>nil then FreeAndNil(f);

  end;
  cube.Free;
  Light.Free;
  Scene.free; //  
  splitList.Free;
  WinMonImu:=nil;
end;

procedure TWinMonImu.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
end;

procedure TWinMonImu.SpeedButton1Click(Sender: TObject);
begin
  if SaveDialog1.Execute then begin
     FileName.Text:=SaveDialog1.FileName;
  end;
end;

procedure TWinMonImu.SpeedButton2Click(Sender: TObject);
begin
  if not CheckBox1.Checked then begin
    f:=nil;
    MainForm.AddToOutput('mon on ekf'+#13#10);
    Timer1.Enabled:=true;

  end else
  if Length(FileName.Text)>0 then begin
    if FileExists(FileName.Text) then
      f:= TFileStream.Create(FileName.Text, fmOpenReadWrite	 or fmShareDenyNone )
    else
      f:= TFileStream.Create(FileName.Text, fmCreate or fmShareDenyNone	 );

    f.Size:=0;

    MainForm.AddToOutput('mon on ekf'+#13#10);
    Timer1.Enabled:=true;

  end else ShowMessage('  ');
end;

procedure TWinMonImu.SpeedButton3Click(Sender: TObject);
begin
  if f<>nil then FreeAndNil(f);
  MainForm.AddToOutput('mon off ekf'+#13#10);
  Timer1.Enabled:=false;
end;

procedure TWinMonImu.Timer1Timer(Sender: TObject);
var
   s:string;
   r, p, y: string;
begin
  s:=MainForm.EKF_mon;
  Monitor.Lines.Add(s);
  if f<>nil then begin
    s:=s+#13#10;
    f.Write(s[1],Length(s));
  end;
  while  Monitor.Lines.Count>2000 do Monitor.Lines.Delete(0);




  s:= Stringreplace(Trim(s),' ',#13#10,[rfReplaceAll]);
  SplitList.Text:=s;
  r:=SplitList[1];
  p:=SplitList[2];
  y:=SplitList[3];
  r:= Stringreplace(r,'r=','',[rfReplaceAll]);
  r:= Stringreplace(r,'.',',',[rfReplaceAll]);
  p:= Stringreplace(p,'p=','',[rfReplaceAll]);
  p:= Stringreplace(p,'.',',',[rfReplaceAll]);
  y:= Stringreplace(y,'y=','',[rfReplaceAll]);
  y:= Stringreplace(y,'.',',',[rfReplaceAll]);


  rx := RevPitch * StrToFloat(p);   // -- pitch
  ry:=  RevYaw * StrToFloat(y);  // yaw
  rz:= RevRoll * StrToFloat(r);      // roll

  if  YawOn then
      Tentity(Scene.Entities.Items[0]).Rotate(rx,ry - YawRefPos,rz) //  
  else
    Tentity(Scene.Entities.Items[0]).Rotate(rx,0,rz);

  Scene.Redraw;                                 // ...   


end;

procedure TWinMonImu.CheckBox5Click(Sender: TObject);
begin
  if CheckBox5.Checked then YawOn:=true else YawOn:=false;
end;

procedure TWinMonImu.CheckBox2Click(Sender: TObject);
begin
  if CheckBox2.Checked then RevRoll:=-1 else RevRoll:=1;
end;

procedure TWinMonImu.CheckBox3Click(Sender: TObject);
begin
  if CheckBox3.Checked then RevPitch:=-1 else RevPitch:=1;
end;

procedure TWinMonImu.CheckBox4Click(Sender: TObject);
begin
  if CheckBox4.Checked then RevYaw:=-1 else RevYaw:=1;
end;

procedure TWinMonImu.SpeedButton4Click(Sender: TObject);
begin
    MainForm.AddToOutput('save ref'+#13#10);
end;

procedure TWinMonImu.SpeedButton5Click(Sender: TObject);
begin
    MainForm.AddToOutput('set trimmer_roll 0'+#13#10);
    MainForm.AddToOutput('set trimmer_pitch 0'+#13#10);

end;

end.
